@@ -2,8 +2,8 @@
 
 package App::WHMCSUtils;
 
-our $DATE = '2018-01-26'; # DATE
-our $VERSION = '0.004'; # VERSION
+our $DATE = '2018-01-29'; # DATE
+our $VERSION = '0.005'; # VERSION
 
 use 5.010001;
 use strict;
@@ -401,6 +401,8 @@ _
     },
 };
 sub calc_deferred_revenue {
+    require String::Escape;
+
     my %args = @_;
 
     log_trace "args=%s", \%args;
@@ -462,7 +464,7 @@ _
     for my $i (0..$#rows) {
         my $row = $rows[$i];
         my $label = "(".($i+1)."/".(scalar @rows).
-            ") item#$row->{id} inv#=$row->{invoiceid} datepaid=#$row->{datepaid} amount=$row->{amount} description='$row->{description}'";
+            ") item#$row->{id} inv#=$row->{invoiceid} datepaid=#$row->{datepaid} type=".($row->{type} // '')." amount=$row->{amount} description='".String::Escape::backslash($row->{description})."'";
         log_trace "Processing $label: %s ...", $row;
         $progress->update if $progress;
 
@@ -479,7 +481,7 @@ _
                 $date1 = "$m{y1}-$m{m1}-$m{d1}";
                 $date2 = "$m{y2}-$m{m2}-$m{d2}";
                 if ($date1 gt $date2) {
-                    log_warn "$label: Date1 '$date1' > date2 '$date2' in description '$row->{description}', assuming immediate";
+                    log_warn "$label: Date1 '$date1' > date2 '$date2', assuming immediate";
                     undef $date1; undef $date2;
                     last CHECK_DATE;
                 }
@@ -487,7 +489,7 @@ _
                 if ($date2 lt $date_old_limit) {
                     $row->{category} = 'old';
                     $row->{rev_past} = $row->{amount};
-                    log_warn "$label: Date2 '$date2' is too old (< $date_old_limit), recognizing as past revenue";
+                    log_info "$label: Date2 '$date2' is too old (< $date_old_limit), recognizing as past revenue";
                     next ITEM;
                 }
             }
@@ -506,7 +508,7 @@ _
                 $type = 'Domain';
                 last INFER_TYPE;
             }
-            if ($row->{description} =~ /^(opsi tambahan|\baddon\b)/i && $date1 && $date2) {
+            if ($row->{description} =~ /^(opsi tambahan|addon)\b/i && $date1 && $date2) {
                 $type = 'Addon';
                 last INFER_TYPE;
             }
@@ -552,7 +554,7 @@ _
             goto DEFER;
         }
 
-        if ($type =~ /^(|Invoice|Item|Hosting|Addon|Domain|DomainRegister|DomainTransfer|PromoDomain|PromoHosting|Upgrade|MG_DIS_CHARGE)$/) {
+        if ($type =~ /^(|Invoice|Item|Hosting|Addon|Domain|DomainAddonIDP|DomainRegister|DomainTransfer|PromoDomain|PromoHosting|Upgrade|MG_DIS_CHARGE)$/) {
             $row->{category} = 'revenue_immediate';
             log_debug "$label: Type is '$type', recognized revenue $row->{amount} immediately (not deferred) at date of payment $row->{datepaid}";
             goto DEFER;
@@ -579,6 +581,22 @@ _
         return [500, "There are still errors in the invoice items, please fix first"];
     }
 
+    log_info "Calculating revenues ...";
+    my %totalrow;
+    for my $row (@rows) {
+        for my $k (keys %$row) {
+            if ($k =~ /^rev_(\d{4})_(\d{2})$/) {
+                $totalrow{$k} += $row->{$k};
+            } elsif ($k =~ /^rev_past$/) {
+                $totalrow{$k} += $row->{$k};
+            }
+        }
+    }
+    $totalrow{rev_total_nonpast} = 0;
+    for (grep {/^rev_\d/} keys %totalrow) {
+        $totalrow{rev_total_nonpast} += $totalrow{$_};
+    }
+
     if ($args{full}) {
         log_info "Producing CSV ...";
         $progress->target(2 * @rows);
@@ -592,6 +610,8 @@ _
         }
         push @fields, "rev_past" if delete $months{rev_past};
         push @fields, $_ for sort keys %months;
+        push @fields, "rev_total_nonpast"
+            if exists $totalrow{rev_total_nonpast};
 
         # output rows
         my $fh;
@@ -603,29 +623,26 @@ _
         }
         require Text::CSV_XS;
         my $csv = Text::CSV_XS->new({ binary=>1 });
+
+        # header row
         $csv->combine(@fields);
         print $fh $csv->string, "\n";
+
+        # data row
         for my $row (@rows) {
             $progress->update;
             $csv->combine(map {$row->{$_} // ''} @fields);
             print $fh $csv->string, "\n";
         }
-    }
 
-    log_info "Calculating revenues ...";
-    my %revs; # key = period
-    for my $row (@rows) {
-        for my $k (keys %$row) {
-            if ($k =~ /^rev_(\d{4})_(\d{2})$/) {
-                $revs{"$1-$2"} += $row->{$k};
-            } elsif ($k =~ /^rev_past$/) {
-                $revs{'00-past'} += $row->{$k};
-            }
-        }
+        # total row
+        $totalrow{id} = "TOTAL";
+        $csv->combine(map {$totalrow{$_} // ''} @fields);
+        print $fh $csv->string, "\n";
     }
 
     $progress->finish if $progress;
-    return [200, "OK", \%revs];
+    return [200, "OK", \%totalrow];
 }
 
 1;
@@ -643,7 +660,7 @@ App::WHMCSUtils - CLI utilities related to WHMCS
 
 =head1 VERSION
 
-This document describes version 0.004 of App::WHMCSUtils (from Perl distribution App-WHMCSUtils), released on 2018-01-26.
+This document describes version 0.005 of App::WHMCSUtils (from Perl distribution App-WHMCSUtils), released on 2018-01-29.
 
 =head1 FUNCTIONS
 
